@@ -748,7 +748,7 @@ static int DoCalRem(ParsePtr p, int col)
     Trigger trig;
     TimeTrig tim;
     Value v;
-    int r;
+    int r, err;
     int jul;
     CalEntry *CurCol = CalColumn[col];
     CalEntry *e;
@@ -763,14 +763,17 @@ static int DoCalRem(ParsePtr p, int col)
     DBufInit(&pre_buf);
 
     /* Parse the trigger date and time */
-    if ( (r=ParseRem(p, &trig, &tim)) ) return r;
+    if ( (r=ParseRem(p, &trig, &tim, 1)) ) return r;
 
 /* Don't include timed reminders in calendar if -a option supplied. */
     if (DontIssueAts && tim.ttime != NO_TIME) return OK;
     if (trig.typ == NO_TYPE) return E_EOLN;
     if (trig.typ == SAT_TYPE) {
 	r=DoSatRemind(&trig, &tim, p);
-	if (r) return r;
+	if (r) {
+	    if (r == E_EXPIRED) return OK;
+	    return r;
+	}
 	if (!LastTrigValid) return OK;
 	r=ParseToken(p, &buf);
 	if (r) return r;
@@ -793,7 +796,7 @@ static int DoCalRem(ParsePtr p, int col)
 	if (!LastTrigValid) return OK;
     } else {
 	/* Calculate the trigger date */
-	jul = ComputeTrigger(trig.scanfrom, &trig, &r);
+	jul = ComputeTrigger(trig.scanfrom, &trig, &r, 1);
 	if (r) return r;
     }
 
@@ -845,7 +848,7 @@ static int DoCalRem(ParsePtr p, int col)
     DBufInit(&obuf);
     if ((jul == JulianToday) ||
 	(DoSimpleCalDelta &&
-	 ShouldTriggerReminder(&trig, &tim, jul))) {
+	 ShouldTriggerReminder(&trig, &tim, jul, &err))) {
 	NumTriggered++;
 
 	if (DoSimpleCalendar || tim.ttime != NO_TIME) {
@@ -871,7 +874,7 @@ static int DoCalRem(ParsePtr p, int col)
 	    char evalBuf[64];
 	    sprintf(evalBuf, "calprefix(%d)", trig.priority);
 	    s2 = evalBuf;
-	    r = EvalExpr(&s2, &v);
+	    r = EvalExpr(&s2, &v, NULL);
 	    if (!r) {
 		if (!DoCoerce(STR_TYPE, &v)) {
 		    if (DBufPuts(&obuf, v.v.str) != OK) {
@@ -908,7 +911,7 @@ static int DoCalRem(ParsePtr p, int col)
 	    char evalBuf[64];
 	    sprintf(evalBuf, "calsuffix(%d)", trig.priority);
 	    s2 = evalBuf;
-	    r = EvalExpr(&s2, &v);
+	    r = EvalExpr(&s2, &v, NULL);
 	    if (!r) {
 		if (!DoCoerce(STR_TYPE, &v)) {
 		    if (DBufPuts(&obuf, v.v.str) != OK) {
@@ -922,7 +925,7 @@ static int DoCalRem(ParsePtr p, int col)
 	    }
 	}
 	s = DBufValue(&obuf);
-	if (!DoSimpleCalendar) while (isspace(*s)) s++;
+	if (!DoSimpleCalendar) while (isempty(*s)) s++;
 	DBufPuts(&pre_buf, s);
 	s = DBufValue(&pre_buf);
 	e = NEW(CalEntry);
@@ -1265,7 +1268,7 @@ static void SortCol(CalEntry **col)
     while (cur->next &&
 	   CompareRems(0, cur->time, cur->priority,
 		       0, cur->next->time, cur->next->priority,
-		       SortByDate, SortByTime, SortByPrio) <= 0) {
+		       SortByDate, SortByTime, SortByPrio, UntimedBeforeTimed) <= 0) {
 	next = cur->next;
 	/* Swap cur and next */
 	if (!prev) {
