@@ -29,12 +29,12 @@
 static char const DontEscapeMe[] =
 "1234567890_-=+abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ@.,";
 
-static int ParseTimeTrig (ParsePtr s, TimeTrig *tim);
+static int ParseTimeTrig (ParsePtr s, TimeTrig *tim, int save_in_globals);
 static int ParseLocalOmit (ParsePtr s, Trigger *t);
 static int ParseScanFrom (ParsePtr s, Trigger *t, int type);
 static int ParsePriority (ParsePtr s, Trigger *t);
 static int ParseUntil (ParsePtr s, Trigger *t);
-static int ShouldTriggerBasedOnWarn (Trigger *t, int jul);
+static int ShouldTriggerBasedOnWarn (Trigger *t, int jul, int *err);
 
 /***************************************************************/
 /*                                                             */
@@ -48,7 +48,7 @@ int DoRem(ParsePtr p)
 
     Trigger trig;
     TimeTrig tim;
-    int r;
+    int r, err;
     int jul;
     DynamicBuffer buf;
     Token tok;
@@ -56,12 +56,20 @@ int DoRem(ParsePtr p)
     DBufInit(&buf);
 
     /* Parse the trigger date and time */
-    if ( (r=ParseRem(p, &trig, &tim)) ) return r;
+    if ( (r=ParseRem(p, &trig, &tim, 1)) ) return r;
 
-    if (trig.typ == NO_TYPE) return E_EOLN;
+    if (trig.typ == NO_TYPE) {
+	PurgeEchoLine("%s\n%s\n", "#!P! Cannot parse next line", CurLine);
+	return E_EOLN;
+    }
     if (trig.typ == SAT_TYPE) {
+	PurgeEchoLine("%s\n", "#!P: Cannot purge SATISFY-type reminders");
+	PurgeEchoLine("%s\n", CurLine);
 	r=DoSatRemind(&trig, &tim, p);
-	if (r) return r;
+	if (r) {
+	    if (r == E_EXPIRED) return OK;
+	    return r;
+	}
 	if (!LastTrigValid) return OK;
 	r=ParseToken(p, &buf);
 	if (r) return r;
@@ -88,12 +96,37 @@ int DoRem(ParsePtr p)
 	trig.typ = tok.val;
 	jul = LastTriggerDate;
 	if (!LastTrigValid) return OK;
+	if (PurgeMode) return OK;
     } else {
 	/* Calculate the trigger date */
-	jul = ComputeTrigger(trig.scanfrom, &trig, &r);
-	if (r) return r;
+	jul = ComputeTrigger(trig.scanfrom, &trig, &r, 1);
+	if (r) {
+	    if (PurgeMode) {
+		PurgeEchoLine("%s: %s\n", "#!P! Problem calculating trigger date", ErrMsg[r]);
+		PurgeEchoLine("%s\n", CurLine);
+	    }
+	    return r;
+	}
     }
 
+    if (PurgeMode) {
+	if (trig.expired || jul < JulianToday) {
+	    if (p->expr_happened) {
+		if (p->nonconst_expr) {
+		    PurgeEchoLine("%s\n", "#!P: Next line may have expired, but contains non-constant expression");
+		    PurgeEchoLine("%s\n", CurLine);
+		} else {
+		    PurgeEchoLine("%s\n", "#!P: Next line has expired, but contains expression...  please verify");
+		    PurgeEchoLine("#!P: Expired: %s\n", CurLine);
+		}
+	    } else {
+		PurgeEchoLine("#!P: Expired: %s\n", CurLine);
+	    }
+	} else {
+	    PurgeEchoLine("%s\n", CurLine);
+	}
+	return OK;
+    }
 /* Queue the reminder, if necessary */
     if (jul == JulianToday &&
 	!(!IgnoreOnce &&
@@ -103,7 +136,7 @@ int DoRem(ParsePtr p)
 /* If we're in daemon mode, do nothing over here */
     if (Daemon) return OK;
 
-    if (ShouldTriggerReminder(&trig, &tim, jul)) {
+    if (ShouldTriggerReminder(&trig, &tim, jul, &err)) {
 	if ( (r=TriggerReminder(p, &trig, &tim, jul)) )
 	    {
 		return r;
@@ -121,12 +154,14 @@ int DoRem(ParsePtr p)
 /*  trigger structure.                                         */
 /*                                                             */
 /***************************************************************/
-int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
+int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 {
     register int r;
     DynamicBuffer buf;
     Token tok;
 
+    int y, m, d;
+
     DBufInit(&buf);
 
     trig->y = NO_YR;
@@ -145,13 +180,16 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
     trig->priority = DefaultPrio;
     trig->sched[0] = 0;
     trig->warn[0] = 0;
+    trig->omitfunc[0] = 0;
     trig->tag[0] = 0;
     trig->passthru[0] = 0;
     tim->ttime = NO_TIME;
     tim->delta = NO_DELTA;
     tim->rep   = NO_REP;
     tim->duration = NO_TIME;
-    LastTriggerTime = NO_TIME;
+    if (save_in_globals) {
+	LastTriggerTime = NO_TIME;
+    }
 
     while(1) {
 	/* Read space-delimited string */
@@ -161,6 +199,32 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
 	/* Figure out what we've got */
 	FindToken(DBufValue(&buf), &tok);
 	switch(tok.type) {
+	case T_Date:
+	    DBufFree(&buf);
+	    if (trig->d != NO_DAY) return E_DAY_TWICE;
+	    if (trig->m != NO_MON) return E_MON_TWICE;
+	    if (trig->y != NO_YR)  return E_YR_TWICE;
+	    FromJulian(tok.val, &y, &m, &d);
+	    trig->y = y;
+	    trig->m = m;
+	    trig->d = d;
+	    break;
+
+	case T_DateTime:
+	    DBufFree(&buf);
+	    if (trig->d != NO_DAY) return E_DAY_TWICE;
+	    if (trig->m != NO_MON) return E_MON_TWICE;
+	    if (trig->y != NO_YR)  return E_YR_TWICE;
+	    FromJulian(tok.val / MINUTES_PER_DAY, &y, &m, &d);
+	    trig->y = y;
+	    trig->m = m;
+	    trig->d = d;
+	    tim->ttime = (tok.val % MINUTES_PER_DAY);
+	    if (save_in_globals) {
+		LastTriggerTime = tim->ttime;
+	    }
+	    break;
+
 	case T_WkDay:
 	    DBufFree(&buf);
 	    if (trig->wd & (1 << tok.val)) return E_WD_TWICE;
@@ -187,7 +251,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
 
 	case T_At:
 	    DBufFree(&buf);
-	    r=ParseTimeTrig(s, tim);
+	    r=ParseTimeTrig(s, tim, save_in_globals);
 	    if (r) return r;
 	    break;
 
@@ -257,6 +321,10 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
 
 	case T_Omit:
 	    DBufFree(&buf);
+	    if (trig->omitfunc[0]) {
+		Eprint("Warning: OMIT is ignored if you use OMITFUNC");
+	    }
+
 	    r = ParseLocalOmit(s, trig);
 	    if (r) return r;
 	    break;
@@ -266,6 +334,16 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
 	    if (trig->scanfrom == NO_DATE) trig->scanfrom = JulianToday;
 	    return OK;
 
+	case T_OmitFunc:
+	    if (trig->localomit) {
+		Eprint("Warning: OMIT is ignored if you use OMITFUNC");
+	    }
+	    r=ParseToken(s, &buf);
+	    if (r) return r;
+	    StrnCpy(trig->omitfunc, DBufValue(&buf), VAR_NAME_LEN);
+	    DBufFree(&buf);
+	    break;
+
 	case T_Warn:
 	    r=ParseToken(s, &buf);
 	    if(r) return r;
@@ -322,7 +400,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
 /*  ParseTimeTrig - parse the AT part of a timed reminder      */
 /*                                                             */
 /***************************************************************/
-static int ParseTimeTrig(ParsePtr s, TimeTrig *tim)
+static int ParseTimeTrig(ParsePtr s, TimeTrig *tim, int save_in_globals)
 {
     Token tok;
     int r;
@@ -354,7 +432,9 @@ static int ParseTimeTrig(ParsePtr s, TimeTrig *tim)
 	    if (tim->ttime == NO_TIME) return E_EXPECT_TIME;
 
 	    /* Save trigger time in global variable */
-	    LastTriggerTime = tim->ttime;
+	    if (save_in_globals) {
+		LastTriggerTime = tim->ttime;
+	    }
 	    PushToken(DBufValue(&buf), s);
 	    DBufFree(&buf);
 	    return OK;
@@ -443,6 +523,23 @@ static int ParseUntil(ParsePtr s, Trigger *t)
 	    d = tok.val;
 	    break;
 
+	case T_Date:
+	    DBufFree(&buf);
+	    if (y != NO_YR) {
+		Eprint("UNTIL: %s", ErrMsg[E_YR_TWICE]);
+		return E_YR_TWICE;
+	    }
+	    if (m != NO_MON) {
+		Eprint("UNTIL: %s", ErrMsg[E_MON_TWICE]);
+		return E_MON_TWICE;
+	    }
+	    if (d != NO_DAY) {
+		Eprint("UNTIL: %s", ErrMsg[E_DAY_TWICE]);
+		return E_DAY_TWICE;
+	    }
+	    FromJulian(tok.val, &y, &m, &d);
+	    break;
+
 	default:
 	    if (y == NO_YR || m == NO_MON || d == NO_DAY) {
 		Eprint("UNTIL: %s", ErrMsg[E_INCOMPLETE]);
@@ -518,6 +615,23 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 	    d = tok.val;
 	    break;
 
+	case T_Date:
+	    DBufFree(&buf);
+	    if (y != NO_YR) {
+		Eprint("%s: %s", word, ErrMsg[E_YR_TWICE]);
+		return E_YR_TWICE;
+	    }
+	    if (m != NO_MON) {
+		Eprint("%s: %s", word, ErrMsg[E_MON_TWICE]);
+		return E_MON_TWICE;
+	    }
+	    if (d != NO_DAY) {
+		Eprint("%s: %s", word, ErrMsg[E_DAY_TWICE]);
+		return E_DAY_TWICE;
+	    }
+	    FromJulian(tok.val, &y, &m, &d);
+	    break;
+
 	default:
 	    if (y == NO_YR || m == NO_MON || d == NO_DAY) {
 		Eprint("%s: %s", word, ErrMsg[E_INCOMPLETE]);
@@ -551,7 +665,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 {
     int r, y, m, d;
-    char PrioExpr[25];
+    char PrioExpr[VAR_NAME_LEN+25];
     char tmpBuf[64];
     DynamicBuffer buf, calRow;
     DynamicBuffer pre_buf;
@@ -572,18 +686,24 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
     if (t->typ == PASSTHRU_TYPE && !strcmp(t->passthru, "COLOR")) {
 	/* Strip off three tokens */
 	r = ParseToken(p, &buf);
-	DBufPuts(&pre_buf, DBufValue(&buf));
-	DBufPutc(&pre_buf, ' ');
+	if (!NextMode) {
+	    DBufPuts(&pre_buf, DBufValue(&buf));
+	    DBufPutc(&pre_buf, ' ');
+	}
 	DBufFree(&buf);
 	if (r) return r;
 	r = ParseToken(p, &buf);
-	DBufPuts(&pre_buf, DBufValue(&buf));
-	DBufPutc(&pre_buf, ' ');
+	if (!NextMode) {
+	    DBufPuts(&pre_buf, DBufValue(&buf));
+	    DBufPutc(&pre_buf, ' ');
+	}
 	DBufFree(&buf);
 	if (r) return r;
 	r = ParseToken(p, &buf);
-	DBufPuts(&pre_buf, DBufValue(&buf));
-	DBufPutc(&pre_buf, ' ');
+	if (!NextMode) {
+	    DBufPuts(&pre_buf, DBufValue(&buf));
+	    DBufPutc(&pre_buf, ' ');
+	}
 	DBufFree(&buf);
 	if (r) return r;
 	t->typ = MSG_TYPE;
@@ -667,38 +787,46 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	return OK;
     }
 
-/* Put the substituted string into the substitution buffer */
-    if (UserFuncExists("msgprefix") == 1) {
-	sprintf(PrioExpr, "msgprefix(%d)", t->priority);
-	s = PrioExpr;
-	r = EvalExpr(&s, &v);
-	if (!r) {
-	    if (!DoCoerce(STR_TYPE, &v)) {
-		if (DBufPuts(&buf, v.v.str) != OK) {
-		    DBufFree(&buf);
-		    DestroyValue(v);
-		    return E_NO_MEM;
+    /* Put the substituted string into the substitution buffer */
+
+    /* Don't use msgprefix() on RUN-type reminders */
+    if (t->typ != RUN_TYPE) {
+	if (UserFuncExists("msgprefix") == 1) {
+	    sprintf(PrioExpr, "msgprefix(%d)", t->priority);
+	    s = PrioExpr;
+	    r = EvalExpr(&s, &v, NULL);
+	    if (!r) {
+		if (!DoCoerce(STR_TYPE, &v)) {
+		    if (DBufPuts(&buf, v.v.str) != OK) {
+			DBufFree(&buf);
+			DestroyValue(v);
+			return E_NO_MEM;
+		    }
 		}
+		DestroyValue(v);
 	    }
-	    DestroyValue(v);
 	}
     }
+
     if ( (r=DoSubst(p, &buf, t, tim, jul, NORMAL_MODE)) ) return r;
-    if (UserFuncExists("msgsuffix") == 1) {
-	sprintf(PrioExpr, "msgsuffix(%d)", t->priority);
-	s = PrioExpr;
-	r = EvalExpr(&s, &v);
-	if (!r) {
-	    if (!DoCoerce(STR_TYPE, &v)) {
-		if (DBufPuts(&buf, v.v.str) != OK) {
-		    DBufFree(&buf);
-		    DestroyValue(v);
-		    return E_NO_MEM;
+    if (t->typ != RUN_TYPE) {
+	if (UserFuncExists("msgsuffix") == 1) {
+	    sprintf(PrioExpr, "msgsuffix(%d)", t->priority);
+	    s = PrioExpr;
+	    r = EvalExpr(&s, &v, NULL);
+	    if (!r) {
+		if (!DoCoerce(STR_TYPE, &v)) {
+		    if (DBufPuts(&buf, v.v.str) != OK) {
+			DBufFree(&buf);
+			DestroyValue(v);
+			return E_NO_MEM;
+		    }
 		}
+		DestroyValue(v);
 	    }
-	    DestroyValue(v);
 	}
     }
+
     if ((!MsgCommand && t->typ == MSG_TYPE) || t->typ == MSF_TYPE) {
 	if (DBufPutc(&buf, '\n') != OK) {
 	    DBufFree(&buf);
@@ -744,7 +872,7 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
     DBufFree(&buf);
     NumTriggered++;
     return OK;
-}    
+}
 
 /***************************************************************/
 /*                                                             */
@@ -752,12 +880,13 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 /*                                                             */
 /*  Return 1 if we should trigger a reminder, based on today's */
 /*  date and the trigger.  Return 0 if reminder should not be  */
-/*  triggered.                                                 */
+/*  triggered.  Sets *err non-zero in event of an error.       */
 /*                                                             */
 /***************************************************************/
-int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int jul)
+int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int jul, int *err)
 {
-    int r;
+    int r, omit;
+    *err = 0;
 
     /* Handle the ONCE modifier in the reminder. */
     if (!IgnoreOnce && t->once !=NO_ONCE && FileAccessDate == JulianToday)
@@ -786,7 +915,7 @@ int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int jul)
 		return 1;
 	    }
 	}
-	return ShouldTriggerBasedOnWarn(t, jul);
+	return ShouldTriggerBasedOnWarn(t, jul, err);
     }
 
     /* Move back by delta days, if any */
@@ -795,9 +924,22 @@ int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int jul)
 	    jul = jul + t->delta;
 	else {
 	    r = t->delta;
-	    while(r && jul > JulianToday) {
+	    int iter = 0;
+	    int max = MaxSatIter;
+	    if (max < r*2) max = r*2;
+	    while(iter++ < max) {
+		if (!r || (jul <= JulianToday)) {
+		    break;
+		}
 		jul--;
-		if (!IsOmitted(jul, t->localomit)) r--;
+		*err = IsOmitted(jul, t->localomit, t->omitfunc, &omit);
+		if (*err) return 0;
+		if (!omit) r--;
+	    }
+	    if (iter > max) {
+		*err = E_CANT_TRIG;
+	        Eprint("Delta: Bad OMITFUNC? %s", ErrMsg[E_CANT_TRIG]);
+		return 0;
 	    }
 	}
     }
@@ -824,12 +966,12 @@ int DoSatRemind(Trigger *trig, TimeTrig *tim, ParsePtr p)
     iter = 0;
     jul = trig->scanfrom;
     while (iter++ < MaxSatIter) {
-	jul = ComputeTrigger(jul, trig, &r);
+	jul = ComputeTrigger(jul, trig, &r, 1);
 	if (r) {
 	    if (r == E_CANT_TRIG) return OK; else return r;
 	}
 	if (jul == -1) {
-	    return OK;
+	    return E_EXPIRED;
 	}
 	s = p->pos;
 	r = EvaluateExpr(p, &v);
@@ -953,12 +1095,12 @@ finished:
 /*  function.                                                  */
 /*                                                             */
 /***************************************************************/
-static int ShouldTriggerBasedOnWarn(Trigger *t, int jul)
+static int ShouldTriggerBasedOnWarn(Trigger *t, int jul, int *err)
 {
     char buffer[VAR_NAME_LEN+32];
     int i;
     char const *s;
-    int r;
+    int r, omit;
     Value v;
     int lastReturnVal = 0; /* Silence compiler warning */
 
@@ -970,7 +1112,7 @@ static int ShouldTriggerBasedOnWarn(Trigger *t, int jul)
     for (i=1; ; i++) {
 	sprintf(buffer, "%s(%d)", t->warn, i);
 	s = buffer;
-	r = EvalExpr(&s, &v);
+	r = EvalExpr(&s, &v, NULL);
 	if (r) {
 	    Eprint("%s: `%s': %s", ErrMsg[M_BAD_WARN_FUNC],
 		   t->warn, ErrMsg[r]);
@@ -996,9 +1138,21 @@ static int ShouldTriggerBasedOnWarn(Trigger *t, int jul)
 	    if (JulianToday + v.v.val == jul) return 1;
 	} else {
 	    int j = jul;
-	    while (v.v.val) {
+	    int iter = 0;
+	    int max = MaxSatIter;
+	    if (max < v.v.val * 2) max = v.v.val*2;
+	    while(iter++ <= max) {
 		j--;
-		if (!IsOmitted(j, t->localomit)) v.v.val++;
+		*err = IsOmitted(j, t->localomit, t->omitfunc, &omit);
+		if (*err) return 0;
+		if (!omit) v.v.val++;
+		if (!v.v.val) {
+		    break;
+		}
+	    }
+	    if (iter > max) {
+	        Eprint("Delta: Bad OMITFUNC? %s", ErrMsg[E_CANT_TRIG]);
+	        return 0;
 	    }
 	    if (j == JulianToday) return 1;
 	}
