@@ -246,7 +246,8 @@ EXTERN char *ErrMsg[] =
     "Esperando numero",
     "Funcao ilegal na clausula WARN",
     "Can't convert between time zones",
-    "No files matching *.rem"
+    "No files matching *.rem",
+    "String too long"
 };
 #endif /* MK_GLOBALS */
 
