@@ -7,7 +7,7 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*  Copyright (C) 1992-1998 by David F. Skoll                  */
-/*  Copyright (C) 1999-2000 by Roaring Penguin Software Inc.   */
+/*  Copyright (C) 1999-2010 by Roaring Penguin Software Inc.   */
 /*                                                             */
 /***************************************************************/
 
@@ -70,6 +70,12 @@ int main(int argc, char *argv[])
 	return 0;
     }
 
+    /* Are we purging old reminders?  Then just run through the loop once! */
+    if (PurgeMode) {
+	DoReminders();
+	return 0;
+    }
+
     /* Not doing a calendar.  Do the regular remind loop */
     ShouldCache = (Iterations > 1);
 
@@ -123,6 +129,17 @@ int main(int argc, char *argv[])
     return 0;
 }
 
+void PurgeEchoLine(char const *fmt, ...)
+{
+    va_list argptr;
+    va_start(argptr, fmt);
+    if (PurgeFP != NULL) {
+	(void) vfprintf(PurgeFP, fmt, argptr);
+    }
+    va_end(argptr);
+
+}
+
 /***************************************************************/
 /*                                                             */
 /*  DoReminders                                                */
@@ -136,6 +153,7 @@ static void DoReminders(void)
     Token tok;
     char const *s;
     Parser p;
+    int purge_handled;
 
     if (!UseStdin) {
 	FileAccessDate = GetAccessDate(InitialFile);
@@ -173,23 +191,40 @@ static void DoReminders(void)
 	    ShouldIgnoreLine())
 	{
 	    /*** IGNORE THE LINE ***/
+	    if (PurgeMode) {
+		if (strncmp(CurLine, "#!P", 3)) {
+		    PurgeEchoLine("%s\n", CurLine);
+		}
+	    }
 	}
 	else {
+	    purge_handled = 0;
 	    /* Create a parser to parse the line */
 	    CreateParser(s, &p);
 	    switch(tok.type) {
 
             case T_Empty:
 	    case T_Comment:
+		if (!strncmp(CurLine, "#!P", 3)) {
+		    purge_handled = 1;
+		}
 		break;
 
-	    case T_Rem:     r=DoRem(&p);     break;
+	    case T_Rem:     r=DoRem(&p); purge_handled = 1; break;
 	    case T_ErrMsg:  r=DoErrMsg(&p);  break;
 	    case T_If:      r=DoIf(&p);      break;
 	    case T_IfTrig:  r=DoIfTrig(&p);  break;
 	    case T_Else:    r=DoElse(&p);    break;
 	    case T_EndIf:   r=DoEndif(&p);   break;
-	    case T_Include: r=DoInclude(&p); break;
+	    case T_Include:
+		/* In purge mode, include closes file, so we
+		   need to echo it here! */
+		if (PurgeMode) {
+		    PurgeEchoLine("%s\n", CurLine);
+		}
+		r=DoInclude(&p);
+		purge_handled = 1;
+		break;
 	    case T_Exit:    DoExit(&p);      break;
 	    case T_Flush:   r=DoFlush(&p);   break;
 	    case T_Set:     r=DoSet(&p);     break;
@@ -204,32 +239,42 @@ static void DoReminders(void)
 		    DestroyParser(&p);
 		    CreateParser(s, &p);
 		    r=DoRem(&p);
+		    purge_handled = 1;
 		}
 		break;
 	    case T_Pop:     r=PopOmitContext(&p);     break;
 	    case T_Preserve: r=DoPreserve(&p);  break;
 	    case T_Push:    r=PushOmitContext(&p);    break;
 	    case T_RemType: if (tok.val == RUN_TYPE) {
-		r=DoRun(&p);
+		    r=DoRun(&p);
+		} else {
+		    CreateParser(CurLine, &p);
+		    r=DoRem(&p);
+		    purge_handled = 1;
+		}
 		break;
-	    } else {
-		CreateParser(CurLine, &p);
-		r=DoRem(&p);
-		break;
-	    }
 
 
 	    /* If we don't recognize the command, do a REM by default */
 	    /* Note:  Since the parser hasn't been used yet, we don't */
 	    /* need to destroy it here. */
 
-	    default: CreateParser(CurLine, &p); r=DoRem(&p); break;
+	    default: CreateParser(CurLine, &p); purge_handled = 1; r=DoRem(&p); break;
 
 	    }
 	    if (r && (!Hush || r != E_RUN_DISABLED)) {
 		Eprint("%s", ErrMsg[r]);
 	    }
-
+	    if (PurgeMode) {
+		if (!purge_handled) {
+		    PurgeEchoLine("%s\n", CurLine);
+		} else {
+		    if (r) {
+			PurgeEchoLine("#!P! Could not parse next line: %s\n", ErrMsg[r]);
+			PurgeEchoLine("%s\n", CurLine);
+		    }
+		}
+	    }
 	    /* Destroy the parser - free up resources it may be tying up */
 	    DestroyParser(&p);
 	}
@@ -347,7 +392,7 @@ int ParseChar(ParsePtr p, int *err, int peek)
 	}
 	p->expr_happened = 1;
 	p->pos++;
-	r = EvalExpr(&(p->pos), &val);
+	r = EvalExpr(&(p->pos), &val, p);
 	if (r) {
 	    *err = r;
 	    DestroyParser(p);
@@ -383,7 +428,7 @@ int ParseNonSpaceChar(ParsePtr p, int *err, int peek)
     ch = ParseChar(p, err, 1);
     if (*err) return 0;
 
-    while (isspace(ch)) {
+    while (isempty(ch)) {
 	ParseChar(p, err, 0);   /* Guaranteed to work */
 	ch = ParseChar(p, err, 1);
 	if (*err) return 0;
@@ -407,12 +452,12 @@ int ParseToken(ParsePtr p, DynamicBuffer *dbuf)
 
     c = ParseChar(p, &err, 0);
     if (err) return err;
-    while (c && isspace(c)) {
+    while (c && isempty(c)) {
 	c = ParseChar(p, &err, 0);
 	if (err) return err;
     }
     if (!c) return OK;
-    while (c && !isspace(c)) {
+    while (c && !isempty(c)) {
 	if (DBufPutc(dbuf, c) != OK) {
 	    DBufFree(dbuf);
 	    return E_NO_MEM;
@@ -443,7 +488,7 @@ int ParseIdentifier(ParsePtr p, DynamicBuffer *dbuf)
 
     c = ParseChar(p, &err, 0);
     if (err) return err;
-    while (c && isspace(c)) {
+    while (c && isempty(c)) {
 	c = ParseChar(p, &err, 0);
 	if (err) return err;
     }
@@ -484,13 +529,13 @@ int EvaluateExpr(ParsePtr p, Value *v)
     int r;
 
     if (p->isnested) return E_PARSE_ERR;  /* Can't nest expressions */
-    while (isspace(*p->pos)) (p->pos)++;
+    while (isempty(*p->pos)) (p->pos)++;
     if (!p->pos) return E_PARSE_ERR;      /* Missing expression */
     if (*p->pos == BEG_OF_EXPR) {
 	(p->pos)++;
 	bracketed = 1;
     }
-    r = EvalExpr(&(p->pos), v);
+    r = EvalExpr(&(p->pos), v, p);
     if (r) return r;
     if (bracketed) {
 	if (*p->pos != END_OF_EXPR) return E_MISS_END;
@@ -567,6 +612,7 @@ void CreateParser(char const *s, ParsePtr p)
     p->allownested = 1;
     p->tokenPushed = NULL;
     p->expr_happened = 0;
+    p->nonconst_expr = 0;
     DBufInit(&p->pushedToken);
 }
 
@@ -670,10 +716,15 @@ int DoIf(ParsePtr p)
 	    Eprint("%s", ErrMsg[r]);
 	} else
 	    if ( (v.type != STR_TYPE && v.v.val) ||
-		 (v.type == STR_TYPE && strcmp(v.v.str, "")) )
+		 (v.type == STR_TYPE && strcmp(v.v.str, "")) ) {
 		syndrome = IF_TRUE | BEFORE_ELSE;
-	    else
+	    } else {
 		syndrome = IF_FALSE | BEFORE_ELSE;
+		if (PurgeMode) {
+		    PurgeEchoLine("%s\n", "#!P: The next IF evaluated false...");
+		    PurgeEchoLine("%s\n", "#!P: REM statements in IF block not checked for purging.");
+		}
+	    }
     }
 
     NumIfs++;
@@ -693,6 +744,8 @@ int DoElse(ParsePtr p)
 {
     unsigned syndrome;
 
+    int was_ignoring = ShouldIgnoreLine();
+
     if (!NumIfs) return E_ELSE_NO_IF;
 
     syndrome = IfFlags >> (2 * NumIfs - 2);
@@ -700,6 +753,10 @@ int DoElse(ParsePtr p)
     if ((syndrome & IF_ELSE_MASK) == AFTER_ELSE) return E_ELSE_NO_IF;
 
     IfFlags |= AFTER_ELSE << (2 * NumIfs - 2);
+    if (PurgeMode && ShouldIgnoreLine() && !was_ignoring) {
+	PurgeEchoLine("%s\n", "#!P: The previous IF evaluated true.");
+	PurgeEchoLine("%s\n", "#!P: REM statements in ELSE block not checked for purging");
+    }
     return VerifyEoln(p);
 }
 
@@ -724,7 +781,7 @@ int DoEndif(ParsePtr p)
 /***************************************************************/
 int DoIfTrig(ParsePtr p)
 {
-    int r;
+    int r, err;
     unsigned syndrome;
     Trigger trig;
     TimeTrig tim;
@@ -734,15 +791,20 @@ int DoIfTrig(ParsePtr p)
     if (NumIfs >= IF_NEST) return E_NESTED_IF;
     if (ShouldIgnoreLine()) syndrome = IF_TRUE | BEFORE_ELSE;
     else {
-	if ( (r=ParseRem(p, &trig, &tim)) ) return r;
+	if ( (r=ParseRem(p, &trig, &tim, 1)) ) return r;
 	if (trig.typ != NO_TYPE) return E_PARSE_ERR;
-	jul = ComputeTrigger(trig.scanfrom, &trig, &r);
+	jul = ComputeTrigger(trig.scanfrom, &trig, &r, 1);
 	if (r) syndrome = IF_TRUE | BEFORE_ELSE;
 	else {
-	    if (ShouldTriggerReminder(&trig, &tim, jul))
+	    if (ShouldTriggerReminder(&trig, &tim, jul, &err)) {
 		syndrome = IF_TRUE | BEFORE_ELSE;
-	    else
+	    } else {
 		syndrome = IF_FALSE | BEFORE_ELSE;
+		if (PurgeMode) {
+		    PurgeEchoLine("%s\n", "#!P: The next IFTRIG did not trigger.");
+		    PurgeEchoLine("%s\n", "#!P: REM statements in IFTRIG block not checked for purging.");
+		}
+	    }
 	}
     }
     NumIfs++;
@@ -889,7 +951,7 @@ int DoBanner(ParsePtr p)
     DBufInit(&buf);
     c = ParseChar(p, &err, 0);
     if (err) return err;
-    while (isspace(c)) {
+    while (isempty(c)) {
 	c = ParseChar(p, &err, 0);
 	if (err) return err;
     }
@@ -969,6 +1031,8 @@ void DoExit(ParsePtr p)
     int r;
     Value v;
 
+    if (PurgeMode) return;
+
     r = EvaluateExpr(p, &v);
     if (r || v.type != INT_TYPE) exit(99);
     exit(v.v.val);
@@ -990,6 +1054,8 @@ int DoErrMsg(ParsePtr p)
 
     DynamicBuffer buf;
 
+    if (PurgeMode) return OK;
+
     DBufInit(&buf);
     t.typ = MSG_TYPE;
     tt.ttime = SystemTime(0) / 60;
@@ -997,7 +1063,7 @@ int DoErrMsg(ParsePtr p)
 	return r;
     }
     s = DBufValue(&buf);
-    while (isspace(*s)) s++;
+    while (isempty(*s)) s++;
     fprintf(ErrFp, "%s\n", s);
     DBufFree(&buf);
     return OK;
