@@ -20,6 +20,7 @@
 #include "protos.h"
 #include "globals.h"
 #include "err.h"
+#include "expr.h"
 
 static int BexistsIntArray (int array[], int num, int key);
 static void InsertIntoSortedArray (int *array, int num, int key);
@@ -177,26 +178,57 @@ int PopOmitContext(ParsePtr p)
 /*                                                             */
 /*  IsOmitted                                                  */
 /*                                                             */
-/*  Return non-zero if date is OMITted, zero if it is not.     */
+/*  Set *omit to non-zero if date is omitted, else 0.  Returns */
+/*  OK or an error code.                                       */
 /*                                                             */
 /***************************************************************/
-int IsOmitted(int jul, int localomit)
+int IsOmitted(int jul, int localomit, char const *omitfunc, int *omit)
 {
     int y, m, d;
 
+    /* If we have an omitfunc, we *only* use it and ignore local/global
+       OMITs */
+    if (omitfunc && *omitfunc && UserFuncExists(omitfunc)) {
+	char expr[VAR_NAME_LEN + 32];
+	char const *s;
+	int r;
+	Value v;
+
+	FromJulian(jul, &y, &m, &d);
+	sprintf(expr, "%s('%04d-%02d-%02d')",
+		omitfunc, y, m+1, d);
+	s = expr;
+	r = EvalExpr(&s, &v, NULL);
+	if (r) return r;
+	if (v.type == INT_TYPE && v.v.val != 0) {
+	    *omit = 1;
+	} else {
+	    *omit = 0;
+	}
+	return OK;
+    }
+
     /* Is it omitted because of local omits? */
-    if (localomit & (1 << (jul % 7))) return 1;
+    if (localomit & (1 << (jul % 7))) {
+	*omit = 1;
+	return OK;
+    }
 
     /* Is it omitted because of fully-specified omits? */
-    if (BexistsIntArray(FullOmitArray, NumFullOmits, jul)) return 1;
+    if (BexistsIntArray(FullOmitArray, NumFullOmits, jul)) {
+	*omit = 1;
+	return OK;
+    }
 
-    /* Get the syndrome */
     FromJulian(jul, &y, &m, &d);
-    if (BexistsIntArray(PartialOmitArray, NumPartialOmits, (m << 5) + d))
-	return 1;
+    if (BexistsIntArray(PartialOmitArray, NumPartialOmits, (m << 5) + d)) {
+	*omit = 1;
+	return OK;
+    }
 
     /* Not omitted */
-    return 0;
+    *omit = 0;
+    return OK;
 }
 
 /***************************************************************/
@@ -262,6 +294,14 @@ int DoOmit(ParsePtr p)
 	if ( (r=ParseToken(p, &buf)) ) return r;
 	FindToken(DBufValue(&buf), &tok);
 	switch (tok.type) {
+	case T_Date:
+	    DBufFree(&buf);
+	    if (y != NO_YR) return E_YR_TWICE;
+	    if (m != NO_MON) return E_MON_TWICE;
+	    if (d != NO_DAY) return E_DAY_TWICE;
+	    FromJulian(tok.val, &y, &m, &d);
+	    break;
+
 	case T_Year:
 	    DBufFree(&buf);
 	    if (y != NO_YR) return E_YR_TWICE;
@@ -279,7 +319,7 @@ int DoOmit(ParsePtr p)
 	    if (d != NO_DAY) return E_DAY_TWICE;
 	    d = tok.val;
 	    break;
-	 
+
 	case T_Delta:
 	    DBufFree(&buf);
 	    break;
