@@ -16,9 +16,12 @@
 /* Define a general malloc routine for creating pointers to objects */
 #define NEW(type) (malloc(sizeof(type)))
 
+/* Characters to ignore */
+#define isempty(c) (isspace(c) || ((c) == '\\'))
+
 #include "dynbuf.h"
 
-int CallUserFunc (char const *name, int nargs);
+int CallUserFunc (char const *name, int nargs, ParsePtr p);
 int DoFset (ParsePtr p);
 void ProduceCalendar (void);
 char const *SimpleTime (int tim);
@@ -26,12 +29,13 @@ char const *CalendarTime (int tim, int duration);
 int DoRem (ParsePtr p);
 int DoFlush (ParsePtr p);
 void DoExit (ParsePtr p);
-int ParseRem (ParsePtr s, Trigger *trig, TimeTrig *tim);
+int ParseRem (ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals);
 int TriggerReminder (ParsePtr p, Trigger *t, TimeTrig *tim, int jul);
-int ShouldTriggerReminder (Trigger *t, TimeTrig *tim, int jul);
+int ShouldTriggerReminder (Trigger *t, TimeTrig *tim, int jul, int *err);
 int DoSubst (ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul, int mode);
 int DoSubstFromString (char const *source, DynamicBuffer *dbuf, int jul, int tim);
-int EvalExpr (char const **e, Value *v);
+int ParseLiteralDate (char const **s, int *jul, int *tim);
+int EvalExpr (char const **e, Value *v, ParsePtr p);
 int DoCoerce (char type, Value *v);
 void PrintValue  (Value *v, FILE *fp);
 int CopyValue (Value *dest, const Value *src);
@@ -42,7 +46,7 @@ int IncludeFile (char const *fname);
 int GetAccessDate (char const *file);
 int SetAccessDate (char const *fname, int jul);
 int TopLevel (void);
-int CallFunc (Operator *f, int nargs);
+int CallFunc (BuiltinFunc *f, int nargs);
 void InitRemind (int argc, char const *argv[]);
 void Usage (void);
 int Julian (int year, int month, int day);
@@ -51,7 +55,7 @@ int ParseChar (ParsePtr p, int *err, int peek);
 int ParseToken (ParsePtr p, DynamicBuffer *dbuf);
 int ParseIdentifier (ParsePtr p, DynamicBuffer *dbuf);
 int EvaluateExpr (ParsePtr p, Value *v);
-int Evaluate (char const **s, Var *locals);
+int Evaluate (char const **s, Var *locals, ParsePtr p);
 int FnPopValStack (Value *val);
 void Eprint (char const *fmt, ...);
 void OutputLine (FILE *fp);
@@ -75,14 +79,14 @@ int DoClear (ParsePtr p);
 int DestroyOmitContexts (void);
 int PushOmitContext (ParsePtr p);
 int PopOmitContext (ParsePtr p);
-int IsOmitted (int jul, int localomit);
+int IsOmitted (int jul, int localomit, char const *omitfunc, int *omit);
 int DoOmit (ParsePtr p);
 int QueueReminder (ParsePtr p, Trigger *trig, TimeTrig *tim, char const *sched);
 void HandleQueuedReminders (void);
 char const *FindInitialToken (Token *tok, char const *s);
 void FindToken (char const *s, Token *tok);
 void FindNumericToken (char const *s, Token *t);
-int ComputeTrigger (int today, Trigger *trig, int *err);
+int ComputeTrigger (int today, Trigger *trig, int *err, int save_in_globals);
 char *StrnCpy (char *dest, char const *source, int n);
 int StrMatch (char const *s1, char const *s2, int n);
 int StrinCmp (char const *s1, char const *s2, int n);
@@ -91,7 +95,7 @@ int StrCmpi (char const *s1, char const *s2);
 Var *FindVar (char const *str, int create);
 int DeleteVar (char const *str);
 int SetVar (char const *str, Value *val);
-int GetVarValue (char const *str, Value *val, Var *locals);
+int GetVarValue (char const *str, Value *val, Var *locals, ParsePtr p);
 int DoSet  (Parser *p);
 int DoUnset  (Parser *p);
 int DoDump (ParsePtr p);
@@ -104,7 +108,8 @@ int DoMsgCommand (char const *cmd, char const *msg);
 int ParseNonSpaceChar (ParsePtr p, int *err, int peek);
 unsigned int HashVal (char const *str);
 int DateOK (int y, int m, int d);
-Operator *FindFunc (char const *name, Operator where[], int num);
+Operator *FindOperator (char const *name, Operator where[], int num);
+BuiltinFunc *FindFunc (char const *name, BuiltinFunc where[], int num);
 int InsertIntoSortBuffer (int jul, int tim, char const *body, int typ, int prio);
 void IssueSortedReminders (void);
 int UserFuncExists (char const *fn);
@@ -128,7 +133,8 @@ void LocalToUTC (int locdate, int loctime, int *utcdate, int *utctime);
 void UTCToLocal (int utcdate, int utctime, int *locdate, int *loctime);
 int MoonPhase (int date, int time);
 void HuntPhase (int startdate, int starttim, int phas, int *date, int *time);
-int CompareRems (int dat1, int tim1, int prio1, int dat2, int tim2, int prio2, int bydate, int bytime, int byprio);
+int CompareRems (int dat1, int tim1, int prio1, int dat2, int tim2, int prio2, int bydate, int bytime, int byprio, int untimed_first);
 void SigIntHandler (int d);
 void GotSigInt (void);
 void SynthesizeTag(char *);
+void PurgeEchoLine(char const *fmt, ...);
