@@ -74,6 +74,7 @@ Token TokArray[] = {
     { "november",	3,	T_Month,	10 },
     { "october",	3,	T_Month,	9 },
     { "omit",		3,	T_Omit,		0 },
+    { "omitfunc",       8,      T_OmitFunc,     0 },
     { "once",		3,	T_Once,		0 },
     { "pop-omit-context", 3,	T_Pop,		0 },
     { "preserve",       8,      T_Preserve,     0 },
@@ -145,9 +146,9 @@ char const *FindInitialToken(Token *tok, char const *s)
 
     tok->type = T_Illegal;
 
-    while (isspace(*s)) s++;
+    while (isempty(*s)) s++;
 
-    while (*s && !isspace(*s)) {
+    while (*s && !isempty(*s)) {
 	if (DBufPutc(&buf, *s++) != OK) return s;
     }
 
@@ -255,12 +256,30 @@ void FindToken(char const *s, Token *tok)
 void FindNumericToken(char const *s, Token *t)
 {
     int mult = 1, hour, min;
+    char const *s_orig = s;
 
     t->type = T_Illegal;
     t->val = 0;
     if (isdigit(*s)) {
 	PARSENUM(t->val, s);
 
+	/* If we hit a '-' or a '/', we may have a date or a datetime */
+	if (*s == '-' || *s == '/') {
+	    char const *p = s_orig;
+	    int jul, tim;
+	    if (ParseLiteralDate(&p, &jul, &tim) == OK) {
+		if (*p) return;
+		if (tim == NO_TIME) {
+		    t->type = T_Date;
+		    t->val = jul;
+		    return;
+		}
+		t->type = T_DateTime;
+		t->val = MINUTES_PER_DAY * jul + tim;
+	    }
+	    return;
+	}
+
 	/* If we hit a comma, swallow it.  This allows stuff
 	   like Jan 6, 1998 */
 	if (*s == ',') {
