@@ -30,6 +30,21 @@ typedef struct {
     int (*func)(void);
 } Operator;
 
+/* Structure for passing in Nargs and out RetVal from functions */
+typedef struct {
+    int nargs;
+    Value retval;
+} func_info;
+
+/* Define the type of user-functions */
+typedef struct {
+    char const *name;
+    char minargs;
+    char maxargs;
+    char is_constant;
+    int (*func)(func_info *);
+} BuiltinFunc;
+
 /* Define the structure of a variable */
 typedef struct var {
     struct var *next;
@@ -40,6 +55,7 @@ typedef struct var {
 
 /* A trigger */
 typedef struct {
+    int expired;
     int wd;
     int d;
     int m;
@@ -56,6 +72,7 @@ typedef struct {
     int priority;
     char sched[VAR_NAME_LEN+1];  /* Scheduling function */
     char warn[VAR_NAME_LEN+1];   /* Warning function    */
+    char omitfunc[VAR_NAME_LEN+1]; /* OMITFUNC function */
     char tag[TAG_LEN+1];
     char passthru[PASSTHRU_LEN+1];
 } Trigger;
@@ -80,6 +97,7 @@ typedef struct {
     DynamicBuffer pushedToken;	/* Pushed-back token */
     char const *tokenPushed;	/* NULL if no pushed-back token */
     char expr_happened;         /* Did we encounter an [expression] ? */
+    char nonconst_expr;         /* Did we encounter a non-constant [expression] ? */
 } Parser;
 
 typedef Parser *ParsePtr;  /* Pointer to parser structure */
@@ -132,7 +150,7 @@ enum TokTypes
   T_IfTrig, T_ErrMsg,
   T_Set, T_UnSet, T_Fset, T_Omit, T_Banner, T_Exit,
   T_WkDay,
-  T_Month, T_Time,
+  T_Month, T_Time, T_Date, T_DateTime,
   T_Skip, T_At, T_RemType, T_Until, T_Year, T_Day, T_Rep, T_Delta, T_Back,
   T_Once,
   T_Empty,
@@ -148,7 +166,8 @@ enum TokTypes
   T_Warn,
   T_Tag,
   T_Duration,
-  T_LongTime
+  T_LongTime,
+  T_OmitFunc
 };
 
 /* The structure of a token */
